@@ -1,7 +1,7 @@
 from typing import List, Union
 
 import pandas as pd
-from datetime import datetime, date
+from datetime import datetime, date, timedelta
 
 from sgclimaapiclient.baseapi import BaseAPI
 
@@ -196,7 +196,7 @@ class SGClimaDataAPI(BaseAPI):
         """
         return self._call_json("/zones/")
 
-    def get_zone(self, zone_id: int) -> dict:
+    def get_zone(self, zone_id: str) -> dict:
         """
         Get a single zone based on its identifier.
         \f
@@ -205,7 +205,7 @@ class SGClimaDataAPI(BaseAPI):
 
         return self._call_json("/zones/{id}".format(id=zone_id))
 
-    def get_zone_equipments(self, zone_id: int, equipment_type: str = None) -> dict:
+    def get_zone_equipments(self, zone_id: str, equipment_type: str = None) -> dict:
         """
         Get all equipments of a given zone.
         \f
@@ -213,6 +213,33 @@ class SGClimaDataAPI(BaseAPI):
         """
         params = self._build_params(equipment_type=equipment_type)
         return self._call_json("/zones/{id}/equipments".format(id=zone_id), params=params)
+    
+    def download_equipment_efficiency(
+            self,
+            equipment_id: str,
+            start: Union[datetime, date, str],
+            end: Union[datetime, date, str]
+    ) -> pd.DataFrame:
+        """
+        Get the Zone data for each Parameter given a date/timestamp range.
+        Things to consider about the data:
+
+        * Each column represents a Parameter identified by 'NAMED_PARAMETER:PARAMETER_ID'.
+        * Each row has a timestamp multiple of 5 meaning that for each day there are 288 rows from 00:00 to 23:55. Even
+        if there is no data available for a timestamp the row is returned.
+        \f
+        :param zone_id: Zone id
+        :param start: The from to be requested. Either a date (yyyy-mm-dd) or a timestamp (yyyy-mm-ddTHH:MM:SS).
+        :param end: The to day to be requested. Either a date (yyyy-mm-dd) or a timestamp (yyyy-mm-ddTHH:MM:SS).
+        Example: 2021-01-01 will return data up to 2020-12-31"
+        :param filter_by_pid: Filter the data to only have the pids specified in the query. It is expected a list
+        of pids. Example: [12345,12346,12347].
+        :param filter_by_named_pid: Filter the data to only have the named PIDs specified in the query. It is expected
+        a list of pids names. Example: ["outside_hum_pid","pot_clim_gen_pid","set_point_zone_pid"]
+        """
+        params = self._build_params(start=start, end=end)
+        return self._call_df("/equipments/{id}/efficiency".format(id=equipment_id), params=params)
+
 
     def download_zone_data(
             self,
@@ -220,7 +247,8 @@ class SGClimaDataAPI(BaseAPI):
             start: Union[datetime, date, str],
             end: Union[datetime, date, str],
             filter_by_pid: List[int] = None,
-            filter_by_named_pid: List[str] = None
+            filter_by_named_pid: List[str] = None,
+            partition: bool = False
     ) -> pd.DataFrame:
         """
         Get the Zone data for each Parameter given a date/timestamp range.
@@ -243,10 +271,35 @@ class SGClimaDataAPI(BaseAPI):
             filter_by_pid = ",".join(str(pid) for pid in filter_by_pid)
         if filter_by_named_pid is not None:
             filter_by_named_pid = ",".join(filter_by_named_pid)
+        if partition:
+            if isinstance(start, str):
+                try:
+                    start = datetime.strptime(start, "%Y-%m-%dT%H:%M:%S")
+                except ValueError:
+                    c = datetime.strptime(start, "%Y-%m-%d")
+            if isinstance(end, str):
+                try:
+                    end = datetime.strptime(end, "%Y-%m-%dT%H:%M:%S")
+                except ValueError:
+                    end = datetime.strptime(end, "%Y-%m-%d")
 
-        params = self._build_params(start=start, end=end, filter_by_pid=filter_by_pid,
-                                    filter_by_named_pid=filter_by_named_pid)
-        return self._call_df("/zones/{id}/data/download".format(id=zone_id), params=params)
+            dfs = []
+            found = False
+            c_start = c
+            while not found:
+
+                c_end = c_start + timedelta(days=30)
+                if c_end > end:
+                    c_end = end
+                    found = True
+                params = self._build_params(start=c_start.strftime("%Y-%m-%dT%H:%M:%S"), end=c_end.strftime("%Y-%m-%dT%H:%M:%S"), filter_by_pid=filter_by_pid, filter_by_named_pid=filter_by_named_pid)
+                dfs.append(self._call_df("/zones/{id}/data/download".format(id=zone_id), params=params))
+                c_start = c_end
+            return pd.concat(dfs)
+        else:
+            params = self._build_params(start=start, end=end, filter_by_pid=filter_by_pid,
+                                        filter_by_named_pid=filter_by_named_pid)
+            return self._call_df("/zones/{id}/data/download".format(id=zone_id), params=params)
 
     def download_zone_last_values(
             self,
@@ -353,21 +406,58 @@ class SGClimaDataAPI(BaseAPI):
                 pids.extend(self.extract_pids(v))
         return pids
 
-    def extract_filtered_pids(self, x, tags=None) -> List[str]:
+    def extract_filtered_pids(self, x: Union[dict, List[dict]], tags: List[str] = [], with_tags: bool = False) -> List[str]:
+        """
+        Method that given a set of SGClima entities (Site, Zone) it returns its PIDs in the format of
+        <pid>.
+        :param: x: SGClima entity
+        :param: tags: List of pids to filter. Returned named pid will match with the ones in the list.
+        """
         pids = []
+
+        for k, v in self._extract(x, tags):
+            try:
+                if k in tags:
+                    if with_tags:
+                        pids.append(f"{k}:{str(v)}")
+                    else:
+                        pids.append(str(v))
+            except TypeError:
+                pass
+            except ValueError:
+                pass
+        return pids
+
+    def extract_filtered_named_pids(self, x: Union[dict, List[dict]], tags: List[str] = [], with_tags: bool = False) -> List[str]:
+        """
+        Method that given a set of SGClima entities (Site, Zone) it returns its named PIDs in the format of
+        <named_pid>:<pid>.
+        :param: x: SGClima entity
+        :param: tags: List of named pids to filter. Returned named pid will match with the ones in the list.
+        """
+        pids = []
+        tags_values = [int(tag.split(":")[-1]) for tag in tags]
+        tags_names = [tag.split(":")[0] for tag in tags]
+        for k, v in self._extract(x, tags):
+            try:
+                if k in tags_names and v == tags_values[tags_names.index(k)]:
+                    if with_tags:
+                        pids.append(f"{k}:{str(v)}")
+                    else:
+                        pids.append(str(v))
+            except TypeError:
+                pass
+            except ValueError:
+                pass
+        return pids
+
+    def _extract(self, x, tags):
         if type(x) == dict:
             for k, v in x.items():
                 if k.endswith('_pid'):
-                    try:
-                        if k in tags:
-                            pids.append(str(v))
-                    except TypeError:
-                        pass
-                    except ValueError:
-                        pass
+                    yield k, int(v) if v is not None else None
                 else:
-                    pids.extend(self.extract_filtered_pids(v, tags))
+                    yield from self._extract(v, tags)
         elif type(x) == list:
             for v in x:
-                pids.extend(self.extract_filtered_pids(v, tags))
-        return pids
+                yield from self._extract(v, tags)
