@@ -9,8 +9,8 @@ from sgclimaapiclient.baseapi import BaseAPI
 
 class SGClimaModelsAPI(BaseAPI):
 
-    def __init__(self, token, endpoint='https://models-api.dc.indoorclima.com', verify=True):
-        super().__init__(token, endpoint, verify)
+    def __init__(self, token, endpoint='https://models-api.dc.indoorclima.com'):
+        super().__init__(token, endpoint)
 
     def list_models(self):
         """
@@ -26,14 +26,19 @@ class SGClimaModelsAPI(BaseAPI):
         """
         return self._call_json("/ml/{id}/".format(id=id))
 
-    def predict(self, id, df, version="production"):
+    def predict(self, model_name, df, version="production"):
         params = {"version": version}
         data = df.to_json(orient="split")
         headers = {'Content-Type': 'application/json; format=pandas-split'}
-        df = self._call_df(url="/ml/predict/{id}".format(id=id), params=params, json_payload=data, headers=headers)
+        df = self._call_df(url=f"/ml/{model_name}/predict", params=params, json_payload=data, headers=headers)
         df = df.stack().to_frame().reset_index()[[0]]
         df.columns = ["prediction"]
         return df
 
-
+    def preprocess(self, model_name, df, version="production"):
+        params = {"version": version}
+        data = df.to_json(orient="split")
+        headers = {'Content-Type': 'application/json; format=pandas-split'}
+        df = self._call_df(url=f"/ml/{model_name}/preprocess", params=params, json_payload=data, headers=headers)
+        return df
 
